//
// nvGlutWidgets
//
//  Adaptor classes to integrate the nvWidgets UI library with the GLUT windowing
// toolkit. The adaptors convert native GLUT UI data to native nvWidgets data. All
// adaptor classes are implemented as in-line code in this header. The adaptor
// defaults to using the standard OpenGL paintor implementation.
//
// Author: Ignacio Castano, Samuel Gateau, Evan Hart
// Email: sdkfeedback@nvidia.com
//
// Copyright (c) NVIDIA Corporation. All rights reserved.
////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef NV_GLUT_WIDGETS_H
#define NV_GLUT_WIDGETS_H

#include <GL/glut.h>
#include <nvGLWidgets.h>


namespace nv {


class GlutUIContext {

protected:

    UIContext *_context;
    UIPainter *_painter;
    bool _ownPainter;

public:

    //
    // Default UI constructor
    //
    //  Creates private OpenGL painter
    //////////////////////////////////////////////////////////////////
    GlutUIContext() : _context(0), _painter(0), _ownPainter(true) {
        _painter = new GLUIPainter;
        _context = new UIContext(*_painter);
    }

    //
    // Alternate UI constructor
    //
    //  Allows for overriding the standard painter
    //////////////////////////////////////////////////////////////////
    GlutUIContext(UIPainter& painter) : _context(0), _painter(&painter), _ownPainter(false) {
    }

    //
    // UI  destructor
    //
    //  Destroy painter if it is private
    //////////////////////////////////////////////////////////////////
    ~GlutUIContext() {
        delete _context;
        if (_ownPainter) delete _painter;
    }

    //
    // UI method for processing window size events
    //
    //  Call this method from the glutReshapeFunc callback
    //////////////////////////////////////////////////////////////////
    void reshape(int w, int h) { _context->reshape( w, h); }

    //
    // UI method for processing GLUT mouse button events
    //
    //  Call this method from the glutMouseFunc callback, the
    // modifier parameter maps to glutGetModifiers.
    //////////////////////////////////////////////////////////////////
    void mouse(int button, int state, int modifier, int x, int y) { 
        int modifierMask = 0;

        if ( button == GLUT_LEFT_BUTTON) button = MOUSE_LEFT_BUTTON;
        else if ( button == GLUT_MIDDLE_BUTTON) button = MOUSE_MIDDLE_BUTTON;
        else if ( button == GLUT_RIGHT_BUTTON) button = MOUSE_RIGHT_BUTTON;

        if ( modifier & GLUT_ACTIVE_ALT) modifierMask |= KEY_ALT;
        if ( modifier & GLUT_ACTIVE_SHIFT) modifierMask |= KEY_SHIFT;
        if ( modifier & GLUT_ACTIVE_CTRL) modifierMask |= KEY_CTRL;

        if ( state == GLUT_DOWN) state = MOUSE_DOWN; else state = MOUSE_UP;

        _context->mouse( button, state, modifierMask, x, y);
    }

    //
    // UI method for processing GLUT mouse button events
    //
    //  Call this method from the glutMouseFunc callback, this
    // version defaults to no modfiers.
    //////////////////////////////////////////////////////////////////
    void mouse(int button, int state, int x, int y) { mouse( button, state, 0, x, y); }

    //
    // UI method for processing mouse motion events
    //
    //  Call this method from the glutMotionFunc callback
    //////////////////////////////////////////////////////////////////
    void motion(int x, int y) { _context->motion( x, y); }

    //
    // UI method for processing key events
    //
    //  Call this method from the glutReshapeFunc callback
    //////////////////////////////////////////////////////////////////
    void keyboardDown(unsigned char k, int x, int y) { _context->keyboardDown( k, x, y); }

    //
    // UI method for processing key events
    //
    //  Call this method from the glutReshapeFunc callback
    //////////////////////////////////////////////////////////////////
    void keyboardUp(unsigned char k, int x, int y) { _context->keyboardUp( k, x, y); }
    
    //
    // UI method for processing key events
    //
    //  Call this method from the glutReshapeFunc callback
    //////////////////////////////////////////////////////////////////
	void specialKeyboardDown(int k, int x, int y) { _context->keyboardDown( translateKey(k), x, y); }

    //
    // UI method for processing key events
    //
    //  Call this method from the glutReshapeFunc callback
    //////////////////////////////////////////////////////////////////
    void specialKeyboardUp(int k, int x, int y) { _context->keyboardUp( translateKey(k), x, y); }

    bool isOnFocus() const  { return _context->isOnFocus(); }

    //
    // UI method for entering UI processing mode
    //
    //  This function must be used to begin the UI processing
    //////////////////////////////////////////////////////////////////
    void begin() { _context->begin(); }

    //
    // UI method for leaving UI processing mode
    //
    //  This function must be used to end the UI processing
    //////////////////////////////////////////////////////////////////
    void end() { _context->end(); }

    ////////////////////////////////////////////////////////////////////////////
    //
    //  UI element processing
    //
    // The following methods provide the interface for rendering and querying
    // UI objects. These methods must be called between begin/end.
    ////////////////////////////////////////////////////////////////////////////

    //
    //  UI method for drawing a static text label
    //
    // rect - optionally provides a location and size for the label
    // text - Text to display for the label
    //////////////////////////////////////////////////////////////////
    void doLabel(const Rect & rect, const char * text, int style = 0) { _context->doLabel( rect, text, style); }

    //
    //  UI method for rendering and processing a push button
    //
    // rect - optionally provides a location and size for the button
    // text - text to display on the button
    // state - whether the button is depressed
    //////////////////////////////////////////////////////////////////
    bool doButton(const Rect & rect, const char * text, bool * state = NULL, int style = 0) { return _context->doButton( rect, text, state, style); }

    bool doCheckButton(const Rect & rect, const char * text, bool * state, int style = 0) { return _context->doCheckButton( rect, text, state, style); }
    bool doRadioButton(int index, const Rect & rect, const char * text, int * selected, int style = 0) { return _context->doRadioButton( index, rect, text, selected, style); }

    bool doHorizontalSlider(const Rect & rect, float min, float max, float * value, int style = 0) { return _context->doHorizontalSlider( rect, min, max, value, style); }

    bool doListItem(int index, const Rect & rect, const char * text, int * selected, int style = 0) { return _context->doListItem( index, rect, text, selected, style); }
    bool doListBox(const Rect & rect, int numOptions, const char * options[], int * selected, int style = 0) { return _context->doListBox( rect, numOptions, options, selected, style); }
    bool doComboBox(const Rect & rect, int numOptions, const char * options[], int * selected, int style = 0) { return _context->doComboBox( rect, numOptions, options, selected, style); }

    bool doLineEdit(const Rect & r, char * text, int maxTextLength, int * nbCharsReturned, int style = 0) { return _context->doLineEdit( r, text, maxTextLength, nbCharsReturned, style); }

    void setLayout(Layout layout, int alignment) { _context->setLayout( layout, alignment); }
    
    void beginGroup() { _context->beginGroup(); }
    void endGroup()  { _context->endGroup(); }

    int getCursorX()  { return _context->getCursorX(); }
    int getCursorY()  { return _context->getCursorY(); }

    const KeyState& getMouseState( int button) { return _context->getMouseState(button); }

    //
    //  Translate non-ascii keys from GLUT to nvWidgets
    //////////////////////////////////////////////////////////////////
	unsigned char translateKey( int k )
	{
		switch (k)
		{
		case GLUT_KEY_F1 :
			return KEYBOARD_F1;
		case GLUT_KEY_F2 :
			return KEYBOARD_F2;
		case GLUT_KEY_F3 :
			return KEYBOARD_F3;
		case GLUT_KEY_F4 :
			return KEYBOARD_F4;
		case GLUT_KEY_F5 :
			return KEYBOARD_F5;
		case GLUT_KEY_F6 :
			return KEYBOARD_F6;
		case GLUT_KEY_F7 :
			return KEYBOARD_F7;
		case GLUT_KEY_F8 :
			return KEYBOARD_F8;
		case GLUT_KEY_F9 :
			return KEYBOARD_F9;
		case GLUT_KEY_F10 :
			return KEYBOARD_F10;
		case GLUT_KEY_F11 :
			return KEYBOARD_F11;
		case GLUT_KEY_F12 :
			return KEYBOARD_F12;
		case GLUT_KEY_LEFT :
			return KEYBOARD_LEFT;
		case GLUT_KEY_UP :
			return KEYBOARD_UP;
		case GLUT_KEY_RIGHT :
			return KEYBOARD_RIGHT;
		case GLUT_KEY_DOWN :
			return KEYBOARD_DOWN;
		case GLUT_KEY_PAGE_UP :
			return KEYBOARD_PAGE_UP;
		case GLUT_KEY_PAGE_DOWN :
			return KEYBOARD_PAGE_DOWN;
		case GLUT_KEY_HOME :
			return KEYBOARD_HOME;
		case GLUT_KEY_END :
			return KEYBOARD_END;
		case GLUT_KEY_INSERT :
			return KEYBOARD_INSERT;
		default:
			return 0;
		} 
	}
};

};



#endif